# Generated by Django 2.1.10 on 2019-07-24 09:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0027_layergroup'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='layer',
            name='group',
        ),
    ]
