# Generated by Django 2.0.9 on 2018-11-14 08:06

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0016_auto_20181120_1059'),
    ]

    operations = [
        migrations.AddField(
            model_name='layer',
            name='settings',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
        ),
    ]
